from django.db.models import Count, Q, F


from aparnik.contrib.suit.api.views import AdminListAPIView, ModelAdminSortAPIView
from aparnik.packages.shops.orders.models import Order
from .serializers import BaseModel, BaseModelAdminSerializer


class BaseModelSortAdminAPIView(ModelAdminSortAPIView):

    """
        A view that returns the count of active users in JSON.
        """
    def __init__(self, *args, **kwargs):
        super(BaseModelSortAdminAPIView, self).__init__(*args, **kwargs)
        command_dict = {
            'visit': {
                'label': 'تعداد نمایش',
                'queryset_filter': Q(),
                'annotate_command': {'sort_count': F('visit_count')},
                'key_sort': 'sort_count',
            },
            'bookmark':{
                'label': 'تعداد بوکمارک',
                'queryset_filter': Q(),
                'annotate_command': {'sort_count': Count('bookmark_obj')},
                'key_sort': 'sort_count',
            }
        }

        self.command_dict.update(command_dict)


class BaseModelAdminListAPIView(AdminListAPIView):
    serializer_class = BaseModelAdminSerializer
    queryset = BaseModel.objects.all()

    def get_sort_api(self, request):
        return BaseModelSortAdminAPIView(request=self.request)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-25 19:46


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('buttons', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='button',
            name='background_color',
            field=models.CharField(blank=True, max_length=6, null=True, verbose_name='Background Color'),
        ),
    ]



from django.db.models import Sum, Count, Q, F
from django.db.models.functions import Coalesce

from aparnik.contrib.suit.api.views import AdminListAPIView
from aparnik.contrib.basemodels.api.admin.views import BaseModelSortAdminAPIView
from aparnik.packages.shops.orders.models import Order
from aparnik.packages.educations.progresses.models import ProgressSummary
from .serializers import Teacher, TeacherAdminListSerializer


class TeacherSortAdminAPIView(BaseModelSortAdminAPIView):
    """
        A view that returns the count of active users in JSON.
        """

    def __init__(self, *args, **kwargs):
        super(TeacherSortAdminAPIView, self).__init__(*args, **kwargs)

        command_dict = {

            'users': {
                'label': 'شرکت کاربر در دوره',
                'queryset_filter': Q(),
                'annotate_command': {
                    'sort_count':
                        Count('course__progress_summaries__user_obj')
                },
                'key_sort': 'sort_count',
            },
            'course_bookmark': {
                'label': 'بوکمارک',
                'queryset_filter': Q(),
                'annotate_command': {
                    'sort_count':
                        Count('course__bookmark_obj')
                },
                'key_sort': 'sort_count',
            },
            'course_visit': {
                'label': 'بوکمارک',
                'queryset_filter': Q(),
                'annotate_command': {
                    'sort_count':
                        Count('course__visit_count')
                },
                'key_sort': 'sort_count',
            },
            'course_notify_me': {
                'label': 'مرا با خبر کن',
                'queryset_filter': Q(),
                'annotate_command': {
                    'sort_count':
                        Count('course__notifyme_model')
                },
                'key_sort': 'sort_count',
            },
            'course_qa_replay': {
                'label': 'پاسخ مدرس',
                'queryset_filter': Q(),
                'annotate_command': {
                    'sort_count':
                        Count('course__question_answers_set__parent_obj', filter=Q(
                            Q(course__question_answers_set__parent_obj__isnull=False),
                            Q(course__question_answers_set__user_obj=F('user_obj'))
                        ))
                },
                'key_sort': 'sort_count',
            },

        }
        command_dict.update(ProgressSummary.sort_progress(prefix='course__progress_summaries'))
        command_dict.update(Order.sort_buy(prefix='course__orderitem_set__order_obj'))
        command_dict.update(Order.sort_buy_waiting(prefix='course__orderitem_set__order_obj'))
        command_dict.update(Order.sort_buy_wallet(prefix='course__orderitem_set__order_obj'))
        command_dict.update(Order.sort_buy_bank(prefix='course__orderitem_set__order_obj'))

        self.command_dict.update(command_dict)


class TeacherAdminListAPIView(AdminListAPIView):
    serializer_class = TeacherAdminListSerializer
    queryset = Teacher.objects.all()

    def get_sort_api(self, request):
        return TeacherSortAdminAPIView(request=self.request)



from django.db.models import Sum, Count, Q, F
from django.db.models.functions import Coalesce


from aparnik.contrib.suit.api.views import AdminListAPIView
from aparnik.contrib.suit.api.views import ModelAdminSortAPIView
from aparnik.packages.shops.orders.models import Order
from aparnik.packages.educations.progresses.models import ProgressSummary
from .serializers import User, UserAdminListSerializer


class UserSortAdminAPIView(ModelAdminSortAPIView):

    """
        A view that returns the count of active users in JSON.
        """
    def __init__(self, *args, **kwargs):
        super(UserSortAdminAPIView, self).__init__(*args, **kwargs)

        command_dict = {
            # TODO: read bellow line
            # این گزارش باید بر اساس تاریخ به مدیر گفته شود
            'register_date': {
                'label': 'ثبت نام',
                'queryset_filter': Q(),
                'annotate_command': {'sort_count': F('created_at')},
                'key_sort': 'sort_count',
            },
            'qa': {
                'label': 'پرسش و پاسخ',
                'queryset_filter': Q(),
                'annotate_command': {'sort_count': Count('qa')},
                'key_sort': 'sort_count',
            },
            'reviews': {
                'label': 'نظر',
                'queryset_filter': Q(),
                'annotate_command': {'sort_count': Count('review')},
                'key_sort': 'sort_count',
            },

        }
        command_dict.update(ProgressSummary.sort_progress(prefix='progress_user'))
        command_dict.update(Order.sort_buy(prefix='order'))
        command_dict.update(Order.sort_buy_waiting(prefix='order'))
        command_dict.update(Order.sort_buy_wallet(prefix='order'))
        command_dict.update(Order.sort_buy_bank(prefix='order'))

        self.command_dict.update(command_dict)


class UserAdminListAPIView(AdminListAPIView):
    serializer_class = UserAdminListSerializer
    queryset = User.objects.all()

    def get_sort_api(self, request):
        return UserSortAdminAPIView(request=self.request)



from django.db.models import Count, Q


from aparnik.contrib.suit.api.views import AdminListAPIView
from aparnik.contrib.basemodels.api.admin.views import BaseModelSortAdminAPIView
from aparnik.packages.shops.orders.models import Order
from .serializers import Product, ProductAdminListSerializer


class ProductSortAdminAPIView(BaseModelSortAdminAPIView):

    """
        A view that returns the count of active users in JSON.
        """
    def __init__(self, *args, **kwargs):
        super(ProductSortAdminAPIView, self).__init__(*args, **kwargs)
        command_dict = {

        }

        command_dict.update(Order.sort_buy_waiting(prefix='orderitem_set__order_obj'))
        command_dict.update(Order.sort_buy(prefix='orderitem_set__order_obj'))
        command_dict.update(Order.sort_buy_wallet(prefix='orderitem_set__order_obj'))
        command_dict.update(Order.sort_buy_bank(prefix='orderitem_set__order_obj'))

        self.command_dict.update(command_dict)


class ProductAdminListAPIView(AdminListAPIView):
    serializer_class = ProductAdminListSerializer
    queryset = Product.objects.all()

    def get_sort_api(self, request):
        return ProductSortAdminAPIView(request=self.request)
